// USB-MidiCom -L476 *********************************************************
//
// Simple transparent data converter for STM32L476 Nucleo-64
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Additionally, a number of user defined MIDI messages can be released by
// attached pushbuttons or potentiometers.
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.h", "userinterface.c",
// "usb-otg.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction 
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32l4xx.h"
#include "globaldefines.h"
#include "main.h"


int main(void)    // ********************************************************
// **************************************************************************
{
uint16_t recvword;
uint8_t recvbyte;

    RCC->APB1ENR1 |= RCC_APB1ENR1_PWREN;  //necessary for what??

    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOAEN;  // Enable Port A clock
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOBEN;  // Enable Port B clock
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIOCEN;  // Enable Port B clock
    RCC->AHB2ENR |= RCC_AHB2ENR_GPIODEN;  // Enable Port B clock

//    GPIOA->MODER = 0x28000000;      //all input except PA14,13
    GPIOA->MODER = 0x00000000;      //primarily all input
    GPIOB->MODER = 0x00000000;      //primarily all input
    GPIOC->MODER = 0x00000000;      //primarily all input
    GPIOD->MODER = 0x00000000;      //primarily all input(physically only PD2)

    GPIOA->PUPDR = 0xA402AAAA;      //primarily all inputs pulldown,
                                    //except PA9,10,11,12,13,14
    GPIOB->PUPDR = 0xAAAAAA8A;      //primarily all inputs pulldown
									//PB2 is Vusb bridge,must be HiZ
//	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPD2);	//PB2 is Vusb bridge,must be HiZ

    GPIOC->PUPDR = 0xA6AAAAAA;		//primarily all inputs pulldown
									//but PC13 (user button,IO#17)= pullUp

    GPIOD->PUPDR = 0x00000020;	//primarily PD2 pullDown (only existing one)

    SetSysClock();  //see below at " Misc Low Level Routines"

//STM32L476 port default mode = ANALOG !!
    GPIOA->MODER &= ~(GPIO_MODER_MODE0_1 | GPIO_MODER_MODE0_1);
    GPIOC->MODER |= GPIO_MODER_MODE0_0 ; // Set PortC0 (green LED) as output
    GPIOC->MODER |= GPIO_MODER_MODE1_0 ; // Set PortC1 (red LED) as output
    GPIOC->BSRR =  GPIO_BSRR_BS0 | GPIO_BSRR_BS1 ; //both LEDs ON

//STM32L476 port default mode = ANALOG !!
    GPIOA->MODER &= ~GPIO_MODER_MODE5_1 ; // Set PortA5 (green User LED) as output
    GPIOA->MODER |= GPIO_MODER_MODE5_0 ; // Set PortA5 (green User LED) as output

    SysTick_Config(SystemCoreClock/1000);

    gSYSCOUNT=20;
    gGREEN_LEDCOUNT=0;
    gRED_LEDCOUNT=0;
    gDELAYCOUNT=0;
    gTIMEOUT=0;
    gGUARDTIME=1000;
    gUSB_GUARDTIME=1000;
    gStartSetup = 0;	//init detection of +++ sequence
    gUsbStartSetup = 0;
    gSetupMode = 0 ;
    gFlashChanged=0;
	gNumBase = 1;		//default hex
	gExecOn = 0;
	gExecBlock = 0;
	gTest = 0;			//for debugging only
	gOver = 0;			//for debugging only

//start Independent Watchdog  **************
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x0000FFFF;
    msDelay(2);
    IWDG->PR = 0x00000002;      //resulting timeout ca. 1-2 seconds
    msDelay(2);
    IWDG->KR = 0x0000CCCC;      //start Independent Watchdog

//check DIPSW and set guDeviceClass ****************************
//PortC is already configured as input, i.e. MODEx_0 and MODEx_1=0
//configure PortC 2,3 as pull-up
    GPIOC->PUPDR |=  (GPIO_PUPDR_PUPD2_0 | GPIO_PUPDR_PUPD3_0);
    GPIOC->PUPDR &= ~(GPIO_PUPDR_PUPD2_1 | GPIO_PUPDR_PUPD3_1);
    msDelay(2);
//DIPSW1 (PortC,2) selects between serial Com port and MIDI interface;
//when PortC,2 = high (default): CDC_CLASS
//when PortC,2 = low (DIPSW closed): MIDI_CLASS
    guDeviceClass = CDC_CLASS;
    if ((GPIOC->IDR & m2) == 0)  guDeviceClass = MIDI_CLASS;
//DIPSW2 (PortC,3) selects between transparent- or MIDI input mode
//see below

//init I/O state  *********************
    popFlashPage15();

//init communication  *********************
    initUsart1();
#ifndef STLINK_CUTOFF
    initUsart2();
#endif

	if (guDeviceClass == CDC_CLASS)  {
		msDelay(10);
		sendB('O');
		sendB('k');
		sendB(0x0D);
#ifndef STLINK_CUTOFF
		sendU2B('O');
		sendU2B('k');
		sendU2B(0x0D);
#endif
	}

    msDelay(10);
    guConfigured = 0;
    initUSB();
	gTIMEOUT = 3000;
	while (guConfigured == 0)  {
		if( gTIMEOUT == 0) break;
	}

	configure_GPIO();
	gExecOn = 0;
	for (recvbyte=0;recvbyte <= IO_NUM;recvbyte++) gInState[recvbyte] = 0;
	initADC();
	msDelay(100);
	MsgExecutor();		//is not send because gExecOn is still = 0!
	msDelay(100);
	MsgExecutor();		//avoids a series of initial input messages
	msDelay(10);

//DIPSW2 (PortC,3) selects between transparent- or MIDI input mode
//when MIDI_interface, MIDI msgs are sent in native MIDI format
//when serial COM port, in input mode MIDI msgs are send as ASCII text
    gExecOn = 0;
    if ((GPIOC->IDR & m3) == 0)  gExecOn = 0xFF;
    msDelay(10);
    while(1)  {

/*
//may be used for test of SysTick correctness
		IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds
        GPIOC->BSRR =  GPIO_BSRR_BS0;  //PC0 (User LED) ON again
        msDelay(10);
        GPIOC->BSRR =  GPIO_BSRR_BR0;  //PC0 (User LED) OFF again
        msDelay(5);
*/

//Jumps into user setup mode after +++ is ok. Returns to mainloop when Exit
        if ((guDeviceClass == CDC_CLASS) && gSetupMode) userConfig();

#ifdef STLINK_USB
//start setup mode with 3 consecutive +++ with 1 sec guardtime before & after
//gStartSetup and gUsbStartSetup work as mutual exclude semaphore, too
        recvword = pullUsart1RxBuf();
        if (recvword)  {
            recvbyte = (uint8_t)(recvword & 0x00FF);
//			sendB(recvbyte);
//		}
            if (guDeviceClass == CDC_CLASS) {

                if (gUsbStartSetup == 0)  {
                    if ((gStartSetup == 4) && (gGUARDTIME))  {
                        gStartSetup = 0;
                    }
                    if ((recvbyte == '+') && (gStartSetup == 2)) {
                        gStartSetup = 3;
                        gGUARDTIME = 1000;      //milliseconds
                    }
                    if ((recvbyte == '+') && (gStartSetup == 1)) {
                        gStartSetup = 2;
                        gGUARDTIME = 400;
                    }
                    if ((recvbyte == '+')&&(gStartSetup==0)&&(gGUARDTIME==0)) {
                        gStartSetup = 1;
                        gGUARDTIME = 400;
                    }
                    if (gSetupMode == 0) {
                        sendU2B(recvbyte);
                        if (gStartSetup == 0) gGUARDTIME = 1000;
                    }
                }
                else sendU2B(recvbyte);
            }
            else enCodeMidi((uint8_t)(recvbyte&0x00FF));
        }
        if (guConfigured)  {
            recvword = pullUsart2RxBuf();
            if (recvword)  {
                recvbyte = (uint8_t)(recvword & 0x00FF);
                if (guDeviceClass == CDC_CLASS) {
                    if (gStartSetup == 0)  {
                        if ((gUsbStartSetup == 4) && (gUSB_GUARDTIME)) {
                            gUsbStartSetup = 0;
                        }
                        if((recvbyte == '+') && (gUsbStartSetup == 2))  {
                            gUsbStartSetup = 3;
                            gUSB_GUARDTIME = 1000;      //milliseconds
                        }
                        if ((recvbyte == '+') && (gUsbStartSetup == 1))  {
                            gUsbStartSetup = 2;
                            gUSB_GUARDTIME = 400;
                        }
                        if ((recvbyte =='+')&&(gUsbStartSetup==0)&&(gUSB_GUARDTIME==0)) {
                            gUsbStartSetup = 1;
                            gUSB_GUARDTIME = 400;
                        }
                        if (gSetupMode == 0) {
                            sendB(recvbyte);
                            if (gUsbStartSetup == 0) gUSB_GUARDTIME = 1000;
                        }
                    }
                }
                else  sendB(recvbyte);
            }
        }
#else  //STLINK_CUTOFF or original Nucleo with additional USB on Veroboard
//start setup mode with 3 consecutive +++ with 1 sec guardtime before & after
//gStartSetup and gUsbStartSetup work as mutual exclude semaphore, too
        recvword = pullUsart1RxBuf();
#ifndef STLINK_CUTOFF
		if (recvword == 0)  recvword = pullUsart2RxBuf();
#endif
        if (recvword)  {
            recvbyte = (uint8_t)(recvword & 0x00FF);
//			sendB(recvbyte);
//		}
            if (guDeviceClass == CDC_CLASS) {

                if (gUsbStartSetup == 0)  {
                    if ((gStartSetup == 4) && (gGUARDTIME))  {
                        gStartSetup = 0;
                    }
                    if ((recvbyte == '+') && (gStartSetup == 2)) {
                        gStartSetup = 3;
                        gGUARDTIME = 1000;      //milliseconds
                    }
                    if ((recvbyte == '+') && (gStartSetup == 1)) {
                        gStartSetup = 2;
                        gGUARDTIME = 400;
                    }
                    if ((recvbyte == '+')&&(gStartSetup==0)&&(gGUARDTIME==0)) {
                        gStartSetup = 1;
                        gGUARDTIME = 400;
                    }
                    if (gSetupMode == 0) {
                        sendUsbByte(recvbyte);
                        if (gStartSetup == 0) gGUARDTIME = 1000;
                    }
                }
                else sendUsbByte(recvbyte);
            }
            else enCodeMidi((uint8_t)(recvbyte&0x00FF));
        }
        if (guConfigured)  {
            recvword = pullUsbRxBuf();
            if (recvword)  {
                recvbyte = (uint8_t)(recvword & 0x00FF);
                if (guDeviceClass == CDC_CLASS) {
                    if (gStartSetup == 0)  {
                        if ((gUsbStartSetup == 4) && (gUSB_GUARDTIME)) {
                            gUsbStartSetup = 0;
                        }
                        if((recvbyte == '+') && (gUsbStartSetup == 2))  {
                            gUsbStartSetup = 3;
                            gUSB_GUARDTIME = 1000;      //milliseconds
                        }
                        if ((recvbyte == '+') && (gUsbStartSetup == 1))  {
                            gUsbStartSetup = 2;
                            gUSB_GUARDTIME = 400;
                        }
                        if ((recvbyte =='+')&&(gUsbStartSetup==0)&&(gUSB_GUARDTIME==0)) {
                            gUsbStartSetup = 1;
                            gUSB_GUARDTIME = 400;
                        }
                        if (gSetupMode == 0) {
                            sendB(recvbyte);
  #ifndef STLINK_CUTOFF
							sendU2B(recvbyte);
  #endif
                            if (gUsbStartSetup == 0) gUSB_GUARDTIME = 1000;
                        }
                    }
                }
                else  {
					sendB(recvbyte);
  #ifndef STLINK_CUTOFF
					sendU2B(recvbyte);
  #endif
				}
            }
        }
#endif  //STLINK_USB
    }
}


void configure_GPIO(void)  // ***********************************************
// **************************************************************************
{
uint32_t k;

	for (k= 1; k <= IO_NUM;k++)  configurePort(k);
}


void configurePort(uint32_t number)  // *************************************
// built as subroutine to supply service to "usb-fs-042.c" hardware independent
// **************************************************************************
{
	switch(number)  {
//CN.. starting from 3.3V terminal
#ifdef STLINK_CUTOFF
	case 1:
//init I/O state
//init pull-up resistor for any user accessible digital I/O
//i.e. default digital input stage = '1'
		GPIOA->MODER &= ~GPIO_MODER_MODER2;					//PA2 is I/O#1
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD2_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD2_0;
//when configured as "analog input" initalize differently:
		if ((gFlashMirror.Msg1D & 0x8000) == 0x8000) { 
			GPIOA->MODER |= GPIO_MODER_MODER2;				//analog
			GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPD2);			//HiZ
            GPIOA->ASCR |= GPIO_ASCR_ASC2;
		}
		break;
	case 2:
		GPIOA->MODER &= ~GPIO_MODER_MODER3;					//PA3 is I/O#2
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD3_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD3_0;
		if ((gFlashMirror.Msg2D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER3;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD3;
            GPIOA->ASCR |= GPIO_ASCR_ASC3;
		}
		break;
#endif
	case 3:
		GPIOB->MODER &= ~GPIO_MODER_MODER0;					//PB0 is I/O#3
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD0_1;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD0_0;
		if ((gFlashMirror.Msg3D & 0x8000) == 0x8000) {
			GPIOB->MODER |= GPIO_MODER_MODER0;
			GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD0;
            GPIOB->ASCR |= GPIO_ASCR_ASC0;
		}
		break;
	case 4:
		GPIOA->MODER &= ~GPIO_MODER_MODER4;					//PA4 is I/O#4
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD4_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD4_0;
		if ((gFlashMirror.Msg4D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER4;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD4;
            GPIOA->ASCR |= GPIO_ASCR_ASC4;
		}
		break;
	case 5:
		GPIOA->MODER &= ~GPIO_MODER_MODER1;					//PA1 is I/O#5
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD1_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD1_0;
		if ((gFlashMirror.Msg5D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER1;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD1;
            GPIOA->ASCR |= GPIO_ASCR_ASC1;
		}
		break;
	case 6:
		GPIOA->MODER &= ~GPIO_MODER_MODER0;					//PA0 is I/O#6
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD0_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD0_0;
		if ((gFlashMirror.Msg6D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER0;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD0;
            GPIOA->ASCR |= GPIO_ASCR_ASC0;
		}
		break;
	case 7:				//user I/O 7  (PA15)doesn't support analog input
		GPIOA->MODER &= ~GPIO_MODER_MODER15;				//PA15 is I/O#7
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD15_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD15_0;
		break;
	case 8:				//user I/O 8 (PB7) doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER7 ;				//PB7 is I/O#8
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD7_1;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD7_0;
		break;

//CN.. starting from 3.3V terminal
	case 9:				//user I/O 9  (PB3)doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER3;					//PB3 is I/O#9
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD3;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD3_0;
		break;
	case 10:				//user I/O 10  (PB5)doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER5;					//PB5 is I/O#10
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD5;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD5_0;
		break;
	case 11:				//user I/O 11  (PB4)doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER4;					//PB4 is I/O#11
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD4;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD4_0;
		break;
	case 12:
		GPIOA->MODER &= ~GPIO_MODER_MODER7;					//PA7 is I/O#12
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD7_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD7_0;
		if ((gFlashMirror.Msg12D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER7;				//analog
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD7;				//HiZ
            GPIOA->ASCR |= GPIO_ASCR_ASC7;
		}
		break;
	case 13:
		GPIOA->MODER &= ~GPIO_MODER_MODER6;					//PA6 is I/O#13
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD6_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD6_0;
		if ((gFlashMirror.Msg13D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER6;				//analog
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD6;				//HiZ
            GPIOA->ASCR |= GPIO_ASCR_ASC6;
		}
		break;
	case 14:
		GPIOA->MODER &= ~GPIO_MODER_MODER5;					//PA5 is I/O#14
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD5_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPD5_0;
		if ((gFlashMirror.Msg14D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER5;				//analog
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPD5;				//HiZ
            GPIOA->ASCR |= GPIO_ASCR_ASC5;
		}
		break;
	case 15:				//user I/O 15  (PB9)doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER9;					//PB9 is I/O#15
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD9;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD9_0;
		break;
	case 16:				//user I/O 16  (PB8)doesn't support analog input
		GPIOB->MODER &= ~GPIO_MODER_MODER8;					//PB8 is I/O#16
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPD8;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPD8_0;
		break;
	case 17:				//user I/O 17  (PC13)doesn't support analog input
		GPIOC->MODER &= ~GPIO_MODER_MODER13;	//PC13 (user button) is I/O#17
		GPIOC->PUPDR &= ~GPIO_PUPDR_PUPD13;
		GPIOC->PUPDR |=  GPIO_PUPDR_PUPD13_0;
		break;
	default:
		break;
	}
}

//##########################################################################

// Misc Low Level Routines
//************************

//#ifdef MSI_CLOCK_DIRECT   //  !!!!!!!!  ############################################
//MSI (48MHz) is stabilized by PLL with LSE

static void SetSysClock(void)  // *******************************************
//provides "Vcore range 1", MSI clock 48.000 MHz, no PLL, AHB=SystemClock, APB1+2 = 24 MHz
// **************************************************************************
{
//enable power to the backup domain (LSE oscillator)
    RCC->APB1ENR1 |= RCC_APB1ENR1_PWREN;
//adjust Vcore Range1 and disable backup domain write protection
    while (PWR->SR2 & PWR_SR2_VOSF) ;
    PWR->CR1 &= ~PWR_CR1_VOS_1;
    PWR->CR1 |= (PWR_CR1_VOS_0 | PWR_CR1_DBP);
    while (PWR->SR2 & PWR_SR2_VOSF) ;

//adjust Flash Waitstates and Prefetch for higher System Clock frequency
    FLASH->ACR |= (FLASH_ACR_PRFTEN | FLASH_ACR_LATENCY_2WS); //for SYSCLK=48MHz

    RCC->CR &= ~RCC_CR_PLLON;           //stop PLL, MSI keeps running with default 4MHz
    while(RCC->CR & RCC_CR_PLLRDY) ;    //as a measure of precaution

    RCC->BDCR &= ~0x0000003D;           //reset all LSE relevant bits
//start LSE ocscillator
 //   RCC->BDCR  &= ~RCC_BDCR_LSEDRV;
    RCC->BDCR |= (RCC_BDCR_LSEDRV_0 | RCC_BDCR_LSEDRV_1);     //high drive
    RCC->BDCR |= RCC_BDCR_LSEON;        //start
    while (!(RCC->BDCR & RCC_BDCR_LSERDY));

//adjust MSI oscillator
    while (!(RCC->CR & RCC_CR_MSIRDY)); //as precaution, should be ok
    RCC->CR &= ~RCC_CR_MSIRANGE;        //clear all relevant bits
    RCC->CR |= RCC_CR_MSIRANGE_11;      //set new value (ca.48MHz)
    RCC->CR |= (RCC_CR_MSIRGSEL | RCC_CR_MSIPLLEN | RCC_CR_MSION);
										//synchronize MSI with stable LSE
    while (!(RCC->CR & RCC_CR_MSIRDY)); //confirm PLL lockin at new range

//SYSCLK=MSI and HCLK = SYSCLK, already set in "SystemInit()"
    RCC->CFGR |= (RCC_CFGR_PPRE1_DIV2 | RCC_CFGR_PPRE2_DIV2);
// PCLK1 = HCLK/2 = 24MHz
// PCLK2 = HCLK/2 = 24MHz

// select MSI as clock for USB
    RCC->CCIPR |= RCC_CCIPR_CLK48SEL;    //set CLK48SEL[1:0]=11

    SystemCoreClock = 48000000;

}


void LED_Off(void)  // ******************************************************
// built as subroutine to supply service to "usb-fs-431.c" hardware independent
// **************************************************************************
{

    GPIOC->BSRR = GPIO_BSRR_BR_1;	// clear bit PC1,LED OFF
    gRED_LEDCOUNT = 200;			//milliseconds
}


void msDelay(uint16_t count)   // *******************************************
//blocking delay by 'count' milliseconds
// **************************************************************************
{
    gDELAYCOUNT = count;
    while(gDELAYCOUNT) ;
}


void servSystem(void)   // ************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called in pullUsart1RxBuf, (pullUsart2RxBuf) and pullUsbRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally slowed down by SysTick to 50 times/second
// ****************************************************************************
{
static uint32_t secCount;

    gSYSCOUNT = 20;         //restart 50Hz service interval
//feed watchdog
    IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds

//interpret input and send MIDI messages in native MIDI or ASCII text(CDC_CLASS)
   if (gExecBlock == 0)  MsgExecutor();

//LED control
    if (gGREEN_LEDCOUNT==0)  GPIOC->BSRR =  GPIO_BSRR_BS_0; //PC1 (green LED) ON again
    if (gRED_LEDCOUNT==0) GPIOC->BSRR = GPIO_BSRR_BS_1;     //PC1 (red LED) ON again

	secCount++;
	if (secCount >= 50)  {
		secCount = 0;
		if (gRuntimeSetup.Channel == 0xFFFF) gGlobalMidiChannel = 0;
		else gGlobalMidiChannel = (uint8_t)gRuntimeSetup.Channel;

		if (((GPIOC->IDR & m2) == 0) && (guDeviceClass == CDC_CLASS))  {
			stopUSB();
			guDeviceClass = MIDI_CLASS;
			initUsart1();
			msDelay(500);
			initUSB();
		}
		else if ((GPIOC->IDR & m2) && (guDeviceClass == MIDI_CLASS))  {
			stopUSB();
			guDeviceClass = CDC_CLASS;
			initUsart1();
			msDelay(500);
			initUSB();
		}

		if (gExecBlock == 0)  {
			if (((GPIOC->IDR & m3) == 0) && (gExecOn == 0))  {
				gExecOn = 0xFF;     //input change messages are activated
			}
			if ((GPIOC->IDR & m3) && gExecOn)  {
				gExecOn = 0;		//input change messages deactivated
			}
		}
		else gExecOn = 0;
	}

}


void MsgExecutor(void)   //***********************************************
//read analog inputs PA4,PA1,PA0,PB0 and compare with previous value
//and/or read digital levels of ports A,B,C,D and compare with previous value
//send MIDI messages as configured in gRuntimeSetup
// gRuntimeSetup contains operative data, possibly temporarily updated)
// **************************************************************************
{
uint16_t analogLevel, digitalState;

	if ((ADC1->CR & ADC_CR_ADEN) == 0) initADC();	//repeted for stability
#ifdef STLINK_CUTOFF
	if (gFlashMirror.Msg1D != 0xFFFF) {
			if (((gFlashMirror.Msg1D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(7);           //PA2 is assigned to ADC_IN7
			debounceAnalog(1,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m2);
			debounceDigital(1,digitalState);
		}
	}
	if (gFlashMirror.Msg2D != 0xFFFF) {
			if (((gFlashMirror.Msg2D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(8);           //PA3 is assigned to ADC_IN8
			debounceAnalog(2,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m3);
			debounceDigital(2,digitalState);
		}
	}
#endif
	if (gFlashMirror.Msg3D != 0xFFFF) {
			if (((gFlashMirror.Msg3D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(15);           //PB0 is assigned to ADC_IN15
			debounceAnalog(3,analogLevel);
		}
		else {
			digitalState = (GPIOB->IDR & 1);
			debounceDigital(3,digitalState);
		}
	}
	if (gFlashMirror.Msg4D != 0xFFFF) {
			if (((gFlashMirror.Msg4D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(9);           //PA4 is assigned to ADC_IN9
			debounceAnalog(4,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m4);
			debounceDigital(4,digitalState);
		}
	}
	if (gFlashMirror.Msg5D != 0xFFFF) {
			if (((gFlashMirror.Msg5D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(6);           //PA1 is assigned to ADC_IN6
			debounceAnalog(5,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m1);
			debounceDigital(5,digitalState);
		}
	}
	if (gFlashMirror.Msg6D != 0xFFFF) {
			if (((gFlashMirror.Msg6D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(5);           //PA0 is assigned to ADC_IN5
			debounceAnalog(6,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & 1);
			debounceDigital(6,digitalState);
		}
	}
	if (gFlashMirror.Msg7D != 0xFFFF) {			//PA15
		digitalState = (GPIOA->IDR & m15);
		debounceDigital(7,digitalState);
	}
	if (gFlashMirror.Msg8D != 0xFFFF) {			//PB7
		digitalState = (GPIOB->IDR & m7);
		debounceDigital(8,digitalState);
	}


	if (gFlashMirror.Msg9D != 0xFFFF) {			//PB3
		digitalState = (GPIOB->IDR & m3);
		debounceDigital(9,digitalState);
	}
	if (gFlashMirror.Msg10D != 0xFFFF) {		//PB5
		digitalState = (GPIOB->IDR & m5);
		debounceDigital(10,digitalState);
	}
	if (gFlashMirror.Msg11D != 0xFFFF) {		//PB4
		digitalState = (GPIOB->IDR & m4);
		debounceDigital(11,digitalState);
	}
	if (gFlashMirror.Msg12D != 0xFFFF) {
			if (((gFlashMirror.Msg12D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(12);           //PA7 is assigned to ADC_IN12
			debounceAnalog(12,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m7);
			debounceDigital(12,digitalState);
		}
	}
	if (gFlashMirror.Msg13D != 0xFFFF) {
			if (((gFlashMirror.Msg13D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(11);           //PA6 is assigned to ADC_IN11
			debounceAnalog(13,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m6);
			debounceDigital(13,digitalState);
		}
	}
	if (gFlashMirror.Msg14D != 0xFFFF) {
			if (((gFlashMirror.Msg14D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(10);           //PA5 is assigned to ADC_IN10
			debounceAnalog(14,analogLevel);
		}
		else {
			digitalState = (GPIOA->IDR & m5);
			debounceDigital(14,digitalState);
		}
	}
	if (gFlashMirror.Msg15D != 0xFFFF) {		//PB9
		digitalState = (GPIOB->IDR & m9);
		debounceDigital(15,digitalState);
	}
	if (gFlashMirror.Msg16D != 0xFFFF) {		//PB8
		digitalState = (GPIOB->IDR & m8);
		debounceDigital(16,digitalState);
	}
	if (gFlashMirror.MsgUD != 0xFFFF) {
		digitalState = (GPIOC->IDR & m13);		//User button
		debounceDigital(17,digitalState);
	}
}


void debounceAnalog(uint8_t number, uint16_t level)  // *********************
// **************************************************************************
{
uint16_t * structPtr;
uint16_t hysteresis, statusType;

//if not "high res Pitch Wheel Change" - reduce analogLevel to 7 bit format
//in struct gstructPtr the offset relative to PidH is =(user I/Onumber *2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
	if (*structPtr == 0xFFFF) return ;				//message not defined
	if (*(structPtr+1) == 0xFFFF) return ;			//message not defined

	hysteresis = gRuntimeSetup.Hysteresis;
	statusType = *structPtr;
	if ((statusType & 0x00FF) > 0x000F)  statusType = statusType >> 4;

    if ((statusType != 0x000E)  ||
			((*(structPtr+1) & 0xFF00 )!= 0x8100)) {//different debounce level
		level = level>>5;					//reduce to 7 bit resolution
		if (hysteresis) hysteresis -= 2;	//reduce for low resolution msg
    }

    if (gInTest[number] != level) {
        if (gHysteresis[number])  {     //last change was towards lower level
            if (level < gInTest[number]) {
				gInState[number] = level;
				if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
            }
            if (level > gInTest[number] + hysteresis) {
                                        //direction change,apply hysteresis
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
                gHysteresis[number] = 0;
            }
        }
        else {                          //last change was towards higher level
            if (level > gInTest[number]) {
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
            }
            if (level < gInTest[number] - hysteresis) {
                                        //direction change,apply hysteresis
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
                gHysteresis[number] = 0xFF;
            }
        }
    }
}


void debounceDigital(uint8_t number,uint16_t state)  // *********************
// **************************************************************************
{
uint16_t *structPtr;
uint8_t level;

    if ((state) != gInTest[number])  gInTest[number] = state;
    else {                                  //debounce for one service period
        if (state != gInState[number])  {
            gInState[number] = state;
			if (gExecOn)  sendDigMsg(number);
        }
    }
}


uint16_t readADC(uint8_t inport)  // ****************************************
//because Vdda is filtered very badly, improve result by oversampling/average
//one subroutine call takes about 350us
// **************************************************************************
{
uint32_t  result, correction;
uint8_t k;

//    GPIOC->BSRR = GPIO_BSRR_BR_0;       // green LED OFF for test w. oscilloscope

    result = 0;
    for (k= 0; k<66; k++)  {   //for an exact average, the limit should be 63
    //small stretch-up because some potentiometers cannot scan the full range
        ADC1->ISR = 0x000007FF;         //clear relevant all bits by writing 1
        ADC1->SQR1 = (uint32_t)(inport<<6) ;//number of scanned channels always=1
        ADC1->CR |= ADC_CR_ADSTART;
        while ((ADC1->ISR & ADC_ISR_EOC) == 0) ;
        result += (uint32_t)ADC1->DR;
    }
    correction = result & 0x0000000F;
    result = result >>4;
    if (correction > 0x00000007) result++;  //commercial roundup
    result -= 64;    //compensate potentiometer with residual end resistor
    if  (result > 0xFF000000) result = 0;   //kill compensation underflow
    if (result > 0x00000FFF) result = 0x00000FFF;

//    GPIOC->BSRR = GPIO_BSRR_BS_0;       //PC0 (green LED) ON again

    return (uint16_t)result;
}


void initADC(void)   // *****************************************************
// the voltage filtering of the Nucleo board is very bad
// therefore this ADC can only be used for coarse measurement
// like potentiometers for setting user parameters.
// **************************************************************************
{
    RCC->AHB2ENR |= RCC_AHB2ENR_ADCEN;

    ADC1->CR &= ~ADC_CR_DEEPPWD;        //release from Deep Power Down
    ADC1->CR |= ADC_CR_ADVREGEN;        //enable ADC power regulator
//AVR voltage regulator starting time is regular 20us
    msDelay(2);     //as a precaution

    ADC1->DIFSEL &= 0xFFFF0000;         //force all channels to "single ended"
    ADC123_COMMON->CCR = 2<<16;         //no special ADC channels active
                                //and CKMODE=10:ADC clock source = HLK/2= 12MHz
    msDelay(2);     //as a precaution

    ADC1->ISR |= ADC_ISR_ADRDY;
    ADC1->CR |= ADC_CR_ADEN;
    while ((ADC1->ISR & ADC_ISR_ADRDY) == 0) ;
    ADC1->ISR |= ADC_ISR_ADRDY;

    ADC1->CFGR = 1<<3;                  //right aligned, 10bit resolution

    ADC1->SMPR1 = (3<<27) | (3<<24) | (3<<21) | (3<<18) | (3<<15); //PA4...PA0
    ADC1->SMPR2 = (3<<15) |(3<<6) | (3<<3) | 3 ; //PB0,PA7...PA5
//conversion time for ADC12 = ca.24.5+12.5 cycles about 3us plus overhead
//ch.5(PA0),6(PA1),ch.7(PA2),7(PA3),9(PA4),10(PA5),11(PA6),12(PA7),15(PB0) 

    ADC1->SQR1 = 0;             //always sample only 1 channel, insert Ch# later
    ADC1->SQR2 = 0;
    ADC1->SQR3 = 0;
    ADC1->SQR4 = 0;
	gAdcOn = 0xFF;
}


void sendAnaMsg(uint8_t number)   // ****************************************
// **************************************************************************
{
uint8_t status, data1, data2;
uint16_t value, msgS, msgD;
uint16_t *structPtr;

//in struct gFlashMirror the offset relative to PidH is =(user I/Onumber*2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
	if (*structPtr == 0xFFFF) return;		//no message configured 
    msgS = *structPtr & 0x00FF;
	if (msgS > 0xF3) return;		//1 byte MIDI messages,i.e.bad user setup
    status = (uint8_t)msgS;
    if (status <= 0x0F) status = (status << 4) | gGlobalMidiChannel;
	msgS = status >> 4;						//reduce to status type
	structPtr++;
	msgD = *structPtr;
	if (msgD == 0xFFFF) return;				//no message configured 

    if (guDeviceClass == MIDI_CLASS) sendEncodeB(status);
	
	value = gInState[number];
     if (msgS== 0x000E)  {					//pitch change
		if ((msgD & 0xFF00) == 0x8100)  {	//12 bit analog state message
			data1 = (uint8_t) ((value & 0x001F)<<2);
								//least significant 5 bits->7bit left aligned
			data2 = (uint8_t) (value >> 5);	//=most significant 7bits
			value |= 0x8000;				//discriminator for sendAsciiMsg
		}
		else  {								//7 bit analog state message
			data1 = 0;						//Pitch Wheel LSB=0
			data2 = (uint8_t)(value & 0x007F);
										//reduced to 7bit in calling routine
		}
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1) ;
			sendEncodeB(data2 & 0x7F) ;
		}
		else sendAsciiMsg(status, number, value);
	}
    else if ((msgS < 0x000C) || (status == 0xF2)) {
										//send NotePitch or Controller Num
		data1 = (uint8_t)(*structPtr & 0x007F);
		data2 = (uint8_t)(value & 0x007F);
										//reduced to 7bit in calling routine
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1) ;
			sendEncodeB(data2) ;
		}
		else sendAsciiMsg(status, number,(uint16_t)data2);
    }
    else if ((msgS == 0x000C) || (msgS == 0x000D) || (status == 0xF3)) {
										//send NotePitch or Controller Num
		data2 = (uint8_t)(value & 0x007F);
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data2) ;
		}
		else sendAsciiMsg(status, number,(uint16_t)data2);
    }
	else  {				//measure of precaution,should be filtered off before
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(0);
			sendEncodeB(0);
		}
		else sendAsciiMsg('E', status, 0);
	}
}


void sendDigMsg(uint8_t number)   // ****************************************
// **************************************************************************
{
uint8_t status, data1, data2;
uint16_t  msgS;
uint16_t *structPtr;

//in struct gFlashMirror the offset relative to PidH is=(user I/Onumber *2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
    msgS = (*structPtr++ & 0x00FF);

    if ((msgS >= 0x00F6) && (msgS != 0x00F9) && (msgS != 0x00FD)) {
		if (gInState[number] == 0) {
			//single byte message - send only when pushbuttton is pressed
			if (guDeviceClass == MIDI_CLASS) sendEncodeB(msgS);
			else  sendAsciiMsg(0, number, msgS);
		}
		return;
	}

    status = (uint8_t)msgS;
    if (status <= 0x0F) status = (status << 4) | gGlobalMidiChannel;
    if (guDeviceClass == MIDI_CLASS) sendEncodeB(status);
    msgS = status >> 4;                   //reduce to status type

	if (msgS == 0x000E) data1 = 0;
	else data1 = (uint8_t)(*structPtr & 0x007F);
	if (gInState[number])  data2 = 0;			//button NOT pushed: send 0
	else data2 =(uint8_t)((*structPtr >> 8) & 0x007F); //pushed=config value

	if ((msgS == 0x000C) || (msgS == 0x000D) || (status == 0xF3))  {
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data2);
		}
		else sendAsciiMsg(0,number,data2);	//digital msg with user I/O number
    }
	else if (((msgS >= 0x0009) && (msgS <= 0x000B)) || (msgS == 0x000E) || 
														(status == 0xF2))  {
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1);
			sendEncodeB(data2);
		}
		else sendAsciiMsg(0,number,data2);	//digital msg with user I/O number
    }
	else  {				//measure of precaution,should be filtered off before
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(0);
			sendEncodeB(0);
		}
		else sendAsciiMsg('E', status, 0);
	}
}


void sendAsciiMsg(uint8_t mode, uint8_t number, uint16_t value) // **********
// **************************************************************************
{
const uint8_t errorStr[] = "Error:" ;

	sendAllB(' ');
	if (mode == 0) {
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('d');

		if (value == 0) sendAllB('0');
		else  {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
		}
	}
	else if (mode == 'E') {
		sendAllString(errorStr);
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('E');
		sendAllB('r');
		sendAllB('r');
	}
	else {
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('a');
		if ((mode & 0xF0) == 0xE0) {
			if (value & 0x8000)  {
				sendAllB('h');
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX12(value & 0x0FFF);
			}
			else  sendAllS12(value & 0x0FFF);
			}
			else {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
			}
		}
		else  {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
		}
	}
	sendAllB(' ');
}

// ##########################################################################
//Flash operations  

void popFlashPage15(void)   //***********************************************
//only the first 56 bytes of the page are read
// **************************************************************************
{
uint8_t k;
uint16_t * flashPtr, * fMirrorPtr, * runtimeSetPtr;

    flashPtr = (uint16_t *)PAGE15;
    fMirrorPtr = (uint16_t *) &(gFlashMirror.ComBaud);
    runtimeSetPtr = (uint16_t *) &(gRuntimeSetup.ComBaud);
    for (k=0;k<44;k++)  {
        *fMirrorPtr = *flashPtr;
        *runtimeSetPtr = *flashPtr;
        fMirrorPtr++;
		runtimeSetPtr++;
        flashPtr++;
    }
	if (gFlashMirror.ComBaud == 0xFFFF) {
		gFlashMirror.ComBaud = 11;					//DEfault=115200Baud
		gRuntimeSetup.ComBaud = 11;
	}
	if (gFlashMirror.MidiBaud == 0xFFFF) {
		gFlashMirror.MidiBaud = 31;					//Default=31250Baud
		gRuntimeSetup.MidiBaud = 31;
	}
	if (gFlashMirror.Channel == 0xFFFF) {
		gFlashMirror.Channel = 0;
		gRuntimeSetup.Channel = 0;
	}
	if (gFlashMirror.Hysteresis == 0xFFFF) {
		gFlashMirror.Hysteresis = 0x0004;			//default best for 
		gRuntimeSetup.Hysteresis = 0x0004;			//most applications
	}
	if ((gFlashMirror.VidH == 0xFFFF) || (gFlashMirror.VidL == 0xFFFF)) {
		gFlashMirror.VidH = DEFAULT_VIDH;
		gFlashMirror.VidL = DEFAULT_VIDL;
		gRuntimeSetup.VidH = DEFAULT_VIDH;
		gRuntimeSetup.VidL = DEFAULT_VIDL;
	}
	if ((gFlashMirror.PidH == 0xFFFF) || (gFlashMirror.PidL == 0xFFFF)) {
		gFlashMirror.PidH = DEFAULT_PIDH;
		gFlashMirror.PidL = DEFAULT_PIDL;
		gRuntimeSetup.PidH = DEFAULT_PIDH;
		gRuntimeSetup.PidL = DEFAULT_PIDL;
	}
	if (gFlashMirror.Msg1S == 0xFFFF) {
		gFlashMirror.Msg1S = 0x000B;
		gRuntimeSetup.Msg1S = gFlashMirror.Msg1S;	//default CtrlCh
		gFlashMirror.Msg1D = 0x4001;				//data1=input number
		gRuntimeSetup.Msg1D = 0x4001;				//data2=0x40
	}
	if (gFlashMirror.Msg2S == 0xFFFF) {
		gFlashMirror.Msg2S = 0x000B;
		gRuntimeSetup.Msg2S = gFlashMirror.Msg2S;	//default CtrlCh
		gFlashMirror.Msg2D = 0x4002;				//input number
		gRuntimeSetup.Msg2D = 0x4002;				//data2=0x40
	}
	if (gFlashMirror.Msg3S == 0xFFFF) {
		gFlashMirror.Msg3S = 0x000B;
		gRuntimeSetup.Msg3S = gFlashMirror.Msg3S;	//default CtrlCh
		gFlashMirror.Msg3D = 0x4003;				//input number
		gRuntimeSetup.Msg3D = 0x4003;				//data2=0x40
	}
	if (gFlashMirror.Msg4S == 0xFFFF) {
		gFlashMirror.Msg4S = 0x000B;
		gRuntimeSetup.Msg4S = gFlashMirror.Msg4S;	//default CtrlCh
		gFlashMirror.Msg4D = 0x4004;				//input number
		gRuntimeSetup.Msg4D = 0x4004;				//data2=0x40
	}
	if (gFlashMirror.Msg5S == 0xFFFF) {
		gFlashMirror.Msg5S = 0x000E;
		gRuntimeSetup.Msg5S = gFlashMirror.Msg5S;	//default CtrlCh
		gFlashMirror.Msg5D = 0x8100;				//input number
		gRuntimeSetup.Msg5D = 0x8100;				//data2=0x40
	}
	if (gFlashMirror.Msg6S == 0xFFFF) {
		gFlashMirror.Msg6S = 0x000B;
		gRuntimeSetup.Msg6S = gFlashMirror.Msg6S;	//default CtrlCh
		gFlashMirror.Msg6D = 0x4006;				//input number
		gRuntimeSetup.Msg6D = 0x4006;				//data2=0x40
	}
	if (gFlashMirror.Msg7S == 0xFFFF) {
		gFlashMirror.Msg7S = 0x000B;
		gRuntimeSetup.Msg7S = gFlashMirror.Msg7S;	//default CtrlCh
		gFlashMirror.Msg7D = 0x4007;				//input number
		gRuntimeSetup.Msg7D = 0x4007;				//data2=0x40
	}
	if (gFlashMirror.Msg8S == 0xFFFF) {
		gFlashMirror.Msg8S = 0x000B;
		gRuntimeSetup.Msg8S = gFlashMirror.Msg8S;	//default CtrlCh
		gFlashMirror.Msg8D = 0x4008;				//input number
		gRuntimeSetup.Msg8D = 0x4008;				//data2=0x40
	}
	if (gFlashMirror.Msg9S == 0xFFFF) {
		gFlashMirror.Msg9S = 0x000B;
		gRuntimeSetup.Msg9S = gFlashMirror.Msg9S;	//default CtrlCh
		gFlashMirror.Msg9D = 0x4009;				//input number
		gRuntimeSetup.Msg9D = 0x4009;				//data2=0x40
	}
	if (gFlashMirror.Msg10S == 0xFFFF) {
		gFlashMirror.Msg10S = 0x000B;
		gRuntimeSetup.Msg10S = gFlashMirror.Msg10S;	//default CtrlCh
		gFlashMirror.Msg10D = 0x400A;				//data1=input number
		gRuntimeSetup.Msg10D = 0x400A;				//data2=0x40
	}
	if (gFlashMirror.Msg11S == 0xFFFF) {
		gFlashMirror.Msg11S = 0x000B;
		gRuntimeSetup.Msg11S = gFlashMirror.Msg11S;	//default CtrlCh
		gFlashMirror.Msg11D = 0x400B;				//data1=input number
		gRuntimeSetup.Msg11D = 0x400B;				//data2=0x40
	}
	if (gFlashMirror.Msg12S == 0xFFFF) {
		gFlashMirror.Msg12S = 0x000B;
		gRuntimeSetup.Msg12S = gFlashMirror.Msg12S;	//default CtrlCh
		gFlashMirror.Msg12D = 0x400C;				//input number
		gRuntimeSetup.Msg12D = 0x400C;				//data2=0x40
	}
	if (gFlashMirror.Msg13S == 0xFFFF) {
		gFlashMirror.Msg13S = 0x000E;
		gRuntimeSetup.Msg13S = gFlashMirror.Msg13S;	//default CtrlCh
		gFlashMirror.Msg13D = 0x8000;				//input number
		gRuntimeSetup.Msg13D = 0x8000;				//data2=0x40
	}
	if (gFlashMirror.Msg14S == 0xFFFF) {
		gFlashMirror.Msg14S = 0x000B;
		gRuntimeSetup.Msg14S = gFlashMirror.Msg14S;	//default CtrlCh
		gFlashMirror.Msg14D = 0x400E;				//input number
		gRuntimeSetup.Msg14D = 0x400E;				//data2=0x40
	}
	if (gFlashMirror.Msg15S == 0xFFFF) {
		gFlashMirror.Msg15S = 0x000B;
		gRuntimeSetup.Msg15S = gFlashMirror.Msg15S;	//default CtrlCh
		gFlashMirror.Msg15D = 0x400F;				//input number
		gRuntimeSetup.Msg15D = 0x400F;				//data2=0x40
	}
	if (gFlashMirror.Msg16S == 0xFFFF) {
		gFlashMirror.Msg16S = 0x000B;
		gRuntimeSetup.Msg16S = gFlashMirror.Msg16S;	//default CtrlCh
		gFlashMirror.Msg16D = 0x4010;				//input number
		gRuntimeSetup.Msg16D = 0x4010;				//data2=0x40
	}
	if (gFlashMirror.MsgUS == 0xFFFF) {
		gFlashMirror.MsgUS = 0x000B;
		gRuntimeSetup.MsgUS = gFlashMirror.MsgUS;	//default CtrlCh
		gFlashMirror.MsgUD = 0x4011;				//input number
		gRuntimeSetup.MsgUD = 0x4011;				//data2=0x40
	}
	if (gFlashMirror.NumBase == 0xFFFF) {
		gFlashMirror.NumBase = 1;
		gRuntimeSetup.NumBase = 1;	//default hex
	}
	gGlobalMidiChannel = gFlashMirror.Channel;
	gNumBase = gFlashMirror.NumBase;
    gFlashChanged = 0;
    configure_GPIO();
}


void pushFlashPage15(void)   //*********************************************
//only the first 56 bytes of the page are written
// **************************************************************************
{
uint8_t k;
uint32_t half2word;
uint32_t * flashPtr;
uint16_t * fMirrorPtr;

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
                                //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }

//erase Page15
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;  //wait until flash is not busy
    FLASH->CR = ((15<<3) | FLASH_CR_EOPIE |FLASH_CR_PER);
                                //EOPIE is set formally to get SR_EOP set
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else {
        sendX32(FLASH->SR);        //error management ?
        sendB('!');
        sendB('!');
  #ifndef STLINK_CUTOFF
		sendU2B('!');
		sendU2B('!');
  #endif
    }
    FLASH->CR &= ~FLASH_CR_PER;

//write mirror into Page15
	gFlashMirror.NumBase = (uint16_t)gNumBase;
    flashPtr = (uint32_t *)PAGE15;
    fMirrorPtr = (uint16_t *) &(gFlashMirror.ComBaud);
    FLASH->CR |= FLASH_CR_PG;
// this microcontroller allows exclusively doubleword-wide write in flash
// flash least half-word is first half-word in struct
// this is compatible with flash readout in half-words
    for (k=0;k<22;k++)  {
//1st word into flash buffer
        half2word = (uint32_t)*fMirrorPtr;
        fMirrorPtr++;
        half2word = half2word | (uint32_t)*fMirrorPtr<<16;
        fMirrorPtr++;
        *flashPtr = half2word;
        flashPtr++;
//2nd word into flash buffer
        half2word = (uint32_t)*fMirrorPtr;
        fMirrorPtr++;
        half2word = half2word | (uint32_t)*fMirrorPtr<<16;
        fMirrorPtr++;
        *flashPtr = half2word;
        flashPtr++;
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        else {
            sendX32(FLASH->SR);        //error management ?
            sendB('!');
            sendB('!');
  #ifndef STLINK_CUTOFF
			sendU2B('!');
			sendU2B('!');
  #endif
        }
    }
    FLASH->CR &= ~FLASH_CR_PG;

//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
	gFlashChanged |= 0xFF000000;
}

//###########################################################################

//USART Support
//*************

void initUsart1(void) // ****************************************************
// USART1-TxD is PortB,6
// USART1-RxD is PortA,10
// **************************************************************************
{
uint16_t baudset;

    RCC->APB2ENR |=  RCC_APB2ENR_USART1EN ;   // USART1 clock enable

//modified hard-and firmware version2: PA9 remapped to PB6.
//PA9 connected with Vbus for Vbus detect.
//STM32L476 port default mode = ANALOG !!
	GPIOA->MODER &= ~GPIO_MODER_MODE10_0;   // Set PB6,and PA10 in alternate function mode
	GPIOB->MODER &= ~GPIO_MODER_MODE6_0;    // 10=Alternate function mode
	GPIOA->MODER |= GPIO_MODER_MODE10_1;
	GPIOB->MODER |= GPIO_MODER_MODE6_1;
    //GPIOx->OTYPER is push-pull by default
    //GPIOx->OSPEEDR is 2 MHz by default
    //GPIOx->PUPDR is HiZ by default (except PA13,14,15)

//USART1 is AF7
    GPIOA->AFR[1] &= 0xFFFFF0FF;        //PA10 is RX
    GPIOA->AFR[1] |= 0x00000700;
    GPIOB->AFR[0] &= 0xF0FFFFFF;        //PB6 is TX
    GPIOB->AFR[0] |= 0x07000000;

    gUSART1_RXHEAD = gUSART1_RXBUF;
    gUSART1_RXTAIL = gUSART1_RXBUF;

    if (guDeviceClass == CDC_CLASS)  {
        switch(gFlashMirror.ComBaud)  {
        case 96:
            baudset = 0x09C0;	//9600Baud: 156.0 at 24 MHz APB2 Clock
            break;
        case 19:
            baudset = 0x04E0;	//19200Baud: 78.0 at 24 MHz APB2 Clock
            break;
        case 31:
            baudset = 0x0300;	//MIDI=31250Baud: 48.0 at 24 MHz APB2 Clock
            break;
        case 38:
            baudset = 0x0270;	//38400Baud: 39.0 at 24 MHz APB2 Clock
            break;
        case 57:
            baudset = 0x01A0;	//57600Baud: 26.0 at 24 MHz APB2 Clock
            break;
        case 11:
            baudset = 0x00D0;	//115200Baud: 13.0 at 24 MHz APB2 Cloc
            break;
        default:
            baudset = 0x00D0;	//default (virgin flash)
            break;				//115200Baud: 13.0 at 24 MHz APB2 Cloc
        }
        USART1->BRR = baudset;//default 115200Baud: 17.36 at 24 MHz APB2 Clock
    }
    else {
        switch(gFlashMirror.MidiBaud)  {
        case 96:
            baudset = 0x09C0;	//9600Baud: 156.0 at 24 MHz APB2 Clock
            break;
        case 19:
            baudset = 0x04E0;	//19200Baud: 78.0 at 24 MHz APB2 Clock
            break;
        case 31:
            baudset = 0x0300;	//MIDI=31250Baud: 48.0 at 24 MHz APB2 Clock
            break;
        case 38:
            baudset = 0x0270;	//38400Baud: 39.0 at 24 MHz APB2 Clock
            break;
        case 57:
            baudset = 0x01A0;	//57600Baud: 26.0 at 24 MHz APB2 Clock
            break;
        case 11:
            baudset = 0x00D0;	//115200Baud: 13.0 at 24 MHz APB2 Cloc
            break;
        default:
            baudset = 0x0300;	//default (virgin flash)
            break;				//MIDI=31250Baud: 48.0 at 24 MHz APB2 Clock
        }
        USART1->BRR = baudset;	//default 31250Baud: 48.0 at 24 MHz APB2 Clock
    }
//#endif
	USART1->CR2 = USART_CR2_TXINV;	//1 Stopbit, TX level inverted
//for old hardware with 74HCT00 inverter use:
//    USART1->CR2 = 0;			//default:1 Stopbit
    USART1->CR3 = 0;			//default:no Handshake
    USART1->CR1 =USART_CR1_UE | USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
													//8Bit, no Parity, RX Int

    NVIC_EnableIRQ(USART1_IRQn);
}


void sendB(uint8_t data) // *************************************************
//send 1 raw byte
// **************************************************************************
{
    while(!(USART1->ISR&USART_ISR_TXE));
    USART1->TDR = data;
}

#ifndef STLINK_CUTOFF

void initUsart2(void) // ****************************************************
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
uint16_t baudset;

    RCC->APB1ENR1 |=  RCC_APB1ENR1_USART2EN ;   // USART2 clock enable

//STM32L476 port default mode = ANALOG !!
	GPIOA->MODER &= ~(GPIO_MODER_MODE2_0 |    // Set pin 2,3 in alternate function mode
	                 GPIO_MODER_MODE3_0 ) ;  // 10=Alternate function mode
	GPIOA->MODER |= (GPIO_MODER_MODE2_1 |    // Set pin 2,3 in alternate function mode
	                 GPIO_MODER_MODE3_1 ) ;  // 10=Alternate function mode
    //GPIOA->OTYPER is push-pull by default
    //GPIOA->OSPEEDR is 2 MHz by default
    //GPIOA->PUPDR is HiZ by default (except PA13,14,15)

//USART2 is AF7
    GPIOA->AFR[0] &= 0xFFFF00FF;
    GPIOA->AFR[0] |= 0x00007700;

    gUSART2_RXHEAD = gUSART2_RXBUF;
    gUSART2_RXTAIL = gUSART2_RXBUF;

    baudset = 0x00D0;           //115200Baud: 13.0 at 24 MHz APB1 Clock
    USART2->BRR = baudset;      //only option when used together with onboard ST-Link

    USART2->CR2 = 0;			//default:1 Stopbit
    USART2->CR3 = 0;			//default:kein Handshake
    USART2->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
                                                        //8Bit, no Parity, RX Int
    NVIC_EnableIRQ(USART2_IRQn);
}


void sendU2B(uint8_t data) // ***********************************************
//send 1 raw byte
// **************************************************************************
{
    while(!(USART2->ISR&USART_ISR_TXE));
    USART2->TDR = data;
}


#endif  //STLINK_CUTOFF

void sendAllB(uint8_t data) // **********************************************
// **************************************************************************
{
    if ((gSetupMode == 0x77) || (gSetupMode == 0))  {
		sendB(data);
#ifndef STLINK_CUTOFF
  #ifndef STLINK_USB
		sendU2B(data);
  #endif
#endif
	}
#ifdef STLINK_USB
    if ((gSetupMode == 0xFF ) || (gSetupMode == 0)) sendU2B(data);
#else
    if ((gSetupMode == 0xFF ) || (gSetupMode == 0)) sendUsbByte(data);
#endif
}


void sendEncodeB(uint8_t data) // *******************************************
// **************************************************************************
{
    sendB(data);
#ifndef STLINK_CUTOFF
		sendU2B(data);
#endif
#ifndef STLINK_USB
    enCodeMidi(data);
#endif
}


void sendNibble(uint8_t data) // ********************************************
// internally used to send 1 hex nibble
// **************************************************************************

{
uint8_t temp;

    temp = data+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
#ifndef STLINK_CUTOFF
		sendU2B(data);
#endif
}


void sendAllNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    if ((gSetupMode == 0x77) || (gSetupMode == 0))   {
		sendB(asciiNibble);
#ifndef STLINK_CUTOFF
  #ifndef STLINK_USB
		sendU2B(asciiNibble);
  #endif
#endif
	}
#ifdef STLINK_USB
	if ((gSetupMode == 0xFF) || (gSetupMode == 0)) sendU2B(asciiNibble);
#else
	if ((gSetupMode == 0xFF) || (gSetupMode == 0)) sendUsbByte(asciiNibble);
#endif
}


void sendX8(uint8_t data) // ************************************************
// send 1 byte in hex format (without leading 0x)
// **************************************************************************
{
    uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
#ifndef STLINK_CUTOFF
	sendU2B(temp);
#endif
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;

    sendB(temp);
    sendB(0x20);
#ifndef STLINK_CUTOFF
	sendU2B(temp);
	sendU2B(0x20);
#endif
}


void sendAllX8(uint8_t data) // *********************************************
// send 1 byte in hex format (without leading 0x)
// **************************************************************************
{
    uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendAllB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendAllB(temp);
    sendAllB(0x20);
}


void sendAllX12(uint16_t data) // **********************************************
// send 1 half word in 12 bit hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble;

	if (data > 0x0FFF)  data = 0x0FFF;		//precaution
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendAllNibble(nibble);
    sendAllB(0x20);
}


void sendAllS12(uint16_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading SPACE, but trailing SPACE
//up to  10000
// **************************************************************************
{
uint32_t quotient;

    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;

    quotient = data/1000;
    data = data - quotient*1000;
    sendAllB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendAllB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendAllB((uint8_t)quotient + 0x30);
ONES:
    sendAllB((uint8_t)data + 0x30) ;
    sendAllB(0x20);
}


void sendAllX16(uint16_t data) // *******************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendAllNibble(nibble);
    sendAllB(0x20);
}



void sendX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendB(0x20);
#ifndef STLINK_CUTOFF
	sendU2B(0x20);
#endif

}


void sendAllString(const uint8_t * str) // ***********************************
// ***************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendAllB(byte);
}


void sendCRLF(void)  // *****************************************************
// **************************************************************************
{
    sendAllB(0x0D);
    sendAllB(0x0A);
}


uint32_t pullUsart1RxBuf(void) // ********************************************
// pull 1 byte asynchronously from USART1 reveive buffer
// check first if something received. If yes then set bit 15. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
uint32_t recv;

    if (gSYSCOUNT == 0) servSystem();   //serve watchdog etc.

    if (gUSART1_RXTAIL!=gUSART1_RXHEAD)  {
        recv= (uint32_t)((*gUSART1_RXTAIL++ & 0x000000FF)|0x00008000);
        if (gUSART1_RXTAIL==gUSART1_RXBUFTOP) {
                gUSART1_RXTAIL=gUSART1_RXBUF;
        }
        GPIOC->BSRR = GPIO_BSRR_BR_0; // clear bit PC0, green LED OFF
        gGREEN_LEDCOUNT = 200; //milliseconds
        return recv;
    }
    else return 0;
}

#ifndef STLINK_CUTOFF
uint32_t pullUsart2RxBuf(void) // ********************************************
// pull 1 byte asynchronously from USART2 reveive buffer
// check first if something received.If yes then set bit 14+15.If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
uint32_t recv;

    if (gSYSCOUNT == 0) servSystem();   //serve watchdog etc.

    if (gUSART2_RXTAIL!=gUSART2_RXHEAD)  {
        recv= (uint32_t)((*gUSART2_RXTAIL++ & 0x000000FF)|0x0000C000);
        if (gUSART2_RXTAIL==gUSART2_RXBUFTOP) {
                gUSART2_RXTAIL=gUSART2_RXBUF;
        }
        GPIOC->BSRR = GPIO_BSRR_BR_0; // clear bit PC0, green LED OFF
        gGREEN_LEDCOUNT = 200; //milliseconds
        return recv;
    }
    else return 0;
}
#endif

uint32_t pullAllRxBuf(void)  // *********************************************
//receive byte from all channels in parallel with priority order USB > RS232
// ********************************************************************
{
volatile uint32_t recv;

#ifdef STLINK_USB
  if(((gSetupMode==0xFF)||(gSetupMode==0))&&(recv=pullUsart2RxBuf()))
									return recv; //USB has priority over USART
#else
  if(((gSetupMode==0xFF)||(gSetupMode==0))&&(recv=pullUsbRxBuf())) return recv;
												//USB has priority over USART
#endif
    if((gSetupMode==0x77)||(gSetupMode==0))  {
		if (recv=pullUsart1RxBuf()) return recv;	//has priority over USART2
#ifndef STLINK_CUTOFF
		if (recv=pullUsart2RxBuf()) return recv;
#endif
	}
    return 0;
}

//##########################################################################

// Interrupt Handlers
//*******************

void SysTick_Handler(void)  // ***********************************************
// ***************************************************************************
{
    if ((gSYSCOUNT > 0)&&(gSYSCOUNT < 0x8000)) gSYSCOUNT--; 
									//controls servSystem()
    else gSYSCOUNT = 0;             //double safeiy against underflow

    if (gGREEN_LEDCOUNT > 0) gGREEN_LEDCOUNT--;

    if (gRED_LEDCOUNT > 0) gRED_LEDCOUNT--;

    if (gDELAYCOUNT > 0) gDELAYCOUNT--;

    if (gTIMEOUT > 0) gTIMEOUT--;   //to be used in any subroutine

    if (gGUARDTIME) gGUARDTIME--;
    else {
        if (gStartSetup == 3) {     //device in setup mode
            gStartSetup = 0xFF;
            gUsbStartSetup = 0xFF;
            gSetupMode = 0x77;      //semaphor against competitive access
        }
        else gStartSetup = 0;
    }

    if (gUSB_GUARDTIME) gUSB_GUARDTIME--;
    else {
        if (gUsbStartSetup == 3) {    //host in setup mode

            gStartSetup = 0xFF;
            gUsbStartSetup = 0xFF;
            gSetupMode = 0xFF;      //semaphor against competitive access
        }
        else gUsbStartSetup = 0;
    }
}


void USART1_IRQHandler(void)    // *****************************************
// *************************************************************************
{
    if (USART1->ISR & USART_ISR_RXNE)  {
        *gUSART1_RXHEAD++ = (uint8_t)(USART1->RDR & 0x00FF);
        if (gUSART1_RXHEAD==gUSART1_RXBUFTOP)  gUSART1_RXHEAD=gUSART1_RXBUF;
    }
}

#ifndef STLINK_CUTOFF

void USART2_IRQHandler(void)    // *****************************************
// *************************************************************************
{
    if (USART2->ISR & USART_ISR_RXNE)  {
        *gUSART2_RXHEAD++ = (uint8_t)(USART2->RDR & 0x00FF);
        if (gUSART2_RXHEAD==gUSART2_RXBUFTOP)  gUSART2_RXHEAD=gUSART2_RXBUF;
    }
}

#endif
